#@ ImagePlus (label="the input image stack") INPUT_STACK
#@ ImagePlus (label="the background image") INPUT_BACKGROUND
#@ Integer (label="particle threshold (in gray levels)", value=2000, min=0, max=32768, style="slider") PARTICLE_THRESHOLD

#@output ImagePlus OUTPUT_PLOT
"""This script is supposed to be launched from fiji's jython interpreter
"""


# # note: fiji's jython doesn't support encoding keyword

import sys
print('python version %s' % sys.version)  # prints python version

from lipase.settings import UserSettings

# from lipase import Lipase, ImageLogger
from lipase.imageengine import IImageEngine, PixelType, StackImageFeeder
from lipase.imagej.ijimageengine import IJImageEngine
from lipase.lipase import UserProvidedBackground, GlobulesAreaEstimator

# from ij import IJ  # pylint: disable=import-error
# from ij.gui import GenericDialog, DialogListener # pylint: disable=import-error
# from java.awt.event import ItemListener # pylint: disable=import-error
import ij.gui
from jarray import zeros, array

def run_script():
    global INPUT_STACK  # pylint:disable=global-variable-not-assigned
    global INPUT_BACKGROUND  # pylint:disable=global-variable-not-assigned
    global PARTICLE_THRESHOLD  # pylint:disable=global-variable-not-assigned
    global OUTPUT_PLOT  # pylint:disable=global-variable-not-assigned

    IImageEngine.set_instance(IJImageEngine())

    src_hyperstack = IImageEngine.get_instance().create_hyperstack(width=1, height=1, num_channels=1, num_slices=1, num_frames=1, pixel_type=PixelType.U8)
    src_hyperstack.hyperstack = INPUT_STACK  # pylint: disable=undefined-variable

    src_background = IImageEngine.get_instance().create_image(width=1, height=1, pixel_type=PixelType.U8)
    src_background.ij_image = INPUT_BACKGROUND  # pylint: disable=undefined-variable

    background_estimator = UserProvidedBackground(background_image=src_background)
    processor = GlobulesAreaEstimator(background_estimator=background_estimator, particle_threshold=PARTICLE_THRESHOLD)  # pylint: disable=undefined-variable

    results = processor.detect_particles(src_hyperstack)
    # save_hdf5_file('results.h5', results)
    # results file could be checked with "h5dump --xml ./lipase.git/results.h5"


    x = array(results['frame index'].elements, 'f')
    y = array(results['globules_area_ratio'].elements, 'f')

    plot = ij.gui.Plot('my_title', 'frame', 'globules area ratio', x, y)
    plot.show()
    OUTPUT_PLOT = plot


# note : when launched from fiji, __name__ doesn't have the value "__main__", as when launched from python
run_script()
